
#import <Foundation/Foundation.h>
#import "iCore.h"

@class B4XBitmapWrapper;
@class B4XFont;
@class B4XRect;
@class B4XPath;

//~CustomClassTemplate: filenamewithoutextension:customview, name:Custom View (XUI)
//~version:2.00
//~ObjectWrapper:NSObject*
//~Shortname:B4XView
@interface B4XViewWrapper : B4IObjectWrapper
@property (nonatomic, readonly) int TOUCH_ACTION_DOWN;
@property (nonatomic, readonly) int TOUCH_ACTION_UP;
@property (nonatomic, readonly) int TOUCH_ACTION_MOVE;
@property (nonatomic, readonly) int TOUCH_ACTION_MOVE_NOTOUCH;
//Gets or sets the view's left position.
@property (nonatomic) float Left;
//Gets or sets the view's top position.
@property (nonatomic) float Top;
//Gets or sets the view's width.
@property (nonatomic) float Width;
//Gets or sets the view's height.
@property (nonatomic) float Height;
//Gets or sets whether the view is visible.
@property (nonatomic) BOOL Visible;
//Returns the views parent. The returned view can be uninitialized (if there is no parent).
@property (nonatomic, readonly) B4XViewWrapper *Parent;
//Gets or sets whether the view is enabled. Does nothing if the view does not support this property.
@property (nonatomic) BOOL Enabled;
/**
 * Gets or sets the progress value of ProgressView. The value is scaled between 0 to 100. This is different than the value of the native view.
 *<b>Value should be between 0 to 100</b>.
 * Supported types: ProgressView
 */
@property (nonatomic) int Progress;
@property (nonatomic) NSString *EditTextHint;
@property (nonatomic) NSString *Text;
//Gets or sets the text color. Supported types: TextField, TextView and Label.
@property (nonatomic) int TextColor;

/**
 * Gets or sets the text size. Supported types: TextField, TextView, Button and Label.
 */
@property (nonatomic) float TextSize;
/**
 * Gets or sets the font. Supported types: TextField, TextView, Button and Label.
 */
@property (nonatomic)B4XFont *Font;
//Gets or sets the Switch state. Supported types: Switch.
@property (nonatomic) BOOL Checked;

- (void)SetTextSizeAnimated:(int)Duration1 :(float)TextSize;

- (void)SetTextAlignment:(NSString *)Vertical :(NSString *)Horizontal;

/**
 * Sets the views position and size.
 *Duration - Animation duration in milliseconds. Pass 0 to change it immediately.
 */
- (void)SetLayoutAnimated:(int)Duration :(float)Left :(float)Top :(float)Width :(float)Height;
//Removes the view from its parent.
- (void)RemoveViewFromParent;
//Requests that the view will be focused. Returns True if focus has shifted.
- (BOOL)RequestFocus;
//Fades in or fades out the view.
- (void)SetVisibleAnimated:(int)Duration :(BOOL)Visible;
//Used with a For Each block to iterate over all views recursively.
- (NSObject <B4IIterable> *)GetAllViewsRecursive;
/**
* Loads a layout file. Supported types: Panel
*/
//~RaisesSynchronousEvents:true
- (void)LoadLayout:(NSString *)LayoutFile :(B4I *)bi;
//Gets the view at the specified index. Supported types: Panel.
- (B4XViewWrapper *)GetView:(int)Index;
//Returns the number of direct child views. Supported types: Panel.
@property (nonatomic, readonly)int NumberOfViews;
//Adds a view. Supported types: Panel.
- (void) AddView :(UIView *)View :(float)Left :(float)Top :(float)Width :(float)Height;
//Removes all views. Supported types: Panel.
- (void)RemoveAllViews;
//Captures the views appearance.
- (B4XBitmapWrapper *)Snapshot;
/**
 * Sets the ImageView bitmap. Supported types: ImageView.
 * Also sets the ContentMode to Fit.
 */
- (void)SetBitmap:(UIImage *)Bitmap;

- (B4XBitmapWrapper *)GetBitmap;

//Gets or sets the background color;
@property (nonatomic) int Color;

- (void)SetColorAnimated:(int)Duration1 :(int)FromColor :(int)ToColor;

//Sets the background color and border.
- (void)SetColorAndBorder:(int)BackgroundColor :(float)BorderWidth :(int)BorderColor :(float)BorderCornerRadius;
//Gets or sets the ScrollView vertical offset.
@property (nonatomic) int ScrollViewOffsetY;
//Gets or sets the ScrollView horizontal offset.
@property (nonatomic) int ScrollViewOffsetX;
//Gets the ScrollView inner panel
@property (nonatomic, readonly) B4XViewWrapper* ScrollViewInnerPanel;
//Gets or sets the ScrollView content height.
@property (nonatomic) int ScrollViewContentHeight;
//Gets or sets the ScrollView content width.
@property (nonatomic) int ScrollViewContentWidth;
//Gets or sets the view's tag.
@property (nonatomic) NSObject *Tag;
//Brings the view to front.
- (void)BringToFront;

//Sends the view to the back.
- (void)SendToBack;
- (void)SetRotationAnimated:(int)Duration :(float)Degree;
@property (nonatomic)float Rotation;
@end

//~shortname: B4XBitmap
//~ObjectWrapper:UIImage*
@interface B4XBitmapWrapper : B4IObjectWrapper
@property (nonatomic, readonly) float Width;
@property (nonatomic, readonly) float Height;
@property (nonatomic, readonly) float Scale;
/**
* Writes the image to the output stream. The output stream is not closed.
* Quality - Value between 0 (smaller size, lower quality) to 100 (larger size, higher quality).
* Format - JPEG or PNG.
*/
- (void)WriteToStream:(B4IOutputStream *)OutputStream :(int)Quality :(NSString *)Format;
//Returns a <b>new</b> bitmap with the given width and height.
- (B4XBitmapWrapper *)Resize:(float)Width :(float)Height :(BOOL)KeepAspectRatio;
/**
 * Returns a <b>new</b> rotated bitmap. The bitmap will be rotated clockwise.
 *The following values are supported on all three platforms: 0, 90, 180 and 270.
 */
- (B4XBitmapWrapper *)Rotate:(int)Degrees;
//Returns a <b>new</b> cropped bitmap.
- (B4XBitmapWrapper *)Crop:(float)Left :(float)Top :(float)Width :(float)Height;
@end
/**
 * Cross platform version of the Font object.
 */
//~shortname: B4XFont
//~ObjectWrapper:UIFont*
@interface B4XFont : B4IObjectWrapper
@property (nonatomic, readonly) float Size;

- (B4IFontWrapper *)ToNativeFont;
@end
//~shortname: XUI
@interface B4IXUI : NSObject
@property(nonatomic, readonly) int Color_Black;
@property(nonatomic, readonly) int Color_DarkGray;
@property(nonatomic, readonly) int Color_Gray;
@property(nonatomic, readonly) int Color_LightGray;
@property(nonatomic, readonly) int Color_White;
@property(nonatomic, readonly) int Color_Red;
@property(nonatomic, readonly) int Color_Green;
@property(nonatomic, readonly) int Color_Blue;
@property(nonatomic, readonly) int Color_Yellow;
@property(nonatomic, readonly) int Color_Cyan;
@property(nonatomic, readonly) int Color_Magenta;
@property(nonatomic, readonly) int Color_Transparent;
//Loads a bitmap. In most cases you should use LoadBitmapResize instead.
- (B4XBitmapWrapper *)LoadBitmap:(NSString *)Dir :(NSString *)FileName;
//Loads and resizes a bitmap.
- (B4XBitmapWrapper *)LoadBitmapResize:(NSString *)Dir :(NSString *)FileName :(float)Width :(float)Height :(BOOL)KeepAspectRatio;
//Same as File.DirDocuments.
@property (nonatomic, readonly)NSString *DefaultFolder;
//Does nothing in B4i.
- (void)SetDataFolder:(NSString *)AppName;
//Returns the color value as an Int. Does nothing in B4i.
- (int)PaintOrColorToColor:(NSObject *)Color;
//Same as SubExists keyword.
- (BOOL)SubExists:(NSObject *)Component :(NSString *)Sub :(int)NumberOfArguments;
//Creates a new B4XFont from the given native font and size.
- (B4XFont *)CreateFont:(UIFont *)Font :(float)Size1;
//Creates a new B4XFont from the given font and size.
- (B4XFont *)CreateFont2:(B4XFont *)B4XFont :(float)Size1;
//Creates a new B4XFont with the default font and size.
- (B4XFont *)CreateDefaultFont:(float)Size;
//Creates a new B4XFont with the default bold font and size.
- (B4XFont *)CreateDefaultBoldFont:(float)Size;

- (B4XFont *)CreateFontAwesome:(float)Size1;

- (B4XFont *)CreateMaterialIcons:(float)Size1;

- (B4XViewWrapper *)CreatePanel:(B4I *)bi :(NSString *)EventName;

- (int)Color_RGB:(int)R :(int)G :(int)B;

- (int)Color_ARGB:(int)Alpha :(int)R :(int)G :(int)B;

- (NSObject *)MsgboxAsync:(B4I *)bi :(NSString *)Message :(NSString *)Title;

- (NSObject *)Msgbox2Async:(B4I *)bi :(NSString *)Message :(NSString *)Title :(NSString *)Positive :(NSString *)Cancel :(NSString *)Negative :(UIImage *)Icon;

- (NSString *)FileUri:(NSString *)Dir :(NSString *)FileName;

@property (nonatomic, readonly) int DialogResponse_Positive;
@property (nonatomic, readonly) int DialogResponse_Cancel;
@property (nonatomic, readonly) int DialogResponse_Negative;
@property (nonatomic, readonly) BOOL IsB4A;
@property (nonatomic, readonly) BOOL IsB4i;
@property (nonatomic, readonly) BOOL IsB4J;
@property (nonatomic, readonly) float Scale;
@end
//~shortname: B4XCanvas
@interface B4XCanvas : NSObject
@property (nonatomic, readonly)B4XRect *TargetRect;
@property (nonatomic, readonly)B4XViewWrapper *TargetView;
- (void)Initialize:(UIView *)View;

- (void)Resize:(float)Width :(float)Height;

- (void)Invalidate;

- (B4XBitmapWrapper *)CreateBitmap;

- (void)DrawRect:(B4XRect *)Rect1 :(int)Color :(BOOL)Filled :(float)StrokeWidth;

- (void)DrawLine:(float)x1 :(float)y1 :(float)x2 :(float)y2 :(int)Color :(float)StrokeWidth;

- (void)DrawCircle:(float)x :(float)y :(float)Radius :(int)Color :(BOOL)Filled :(float)StrokeWidth;

- (void)DrawBitmap:(UIImage *)Bitmap :(B4XRect *)Destination;

- (void)DrawBitmapRotated:(UIImage *)Bitmap :(B4XRect *)Destination :(float)Degrees;

- (void)ClipPath:(B4XPath *)Path;

- (void)RemoveClip;

- (void)DrawPath:(B4XPath *)Path :(int)Color :(BOOL)Filled :(float)StrokeWidth;

- (void)DrawPathRotated:(B4XPath *)Path :(int)Color :(BOOL)Filled :(float)StrokeWidth :(float)Degrees :(float)CenterX :(float)CenterY;

- (void)ClearRect:(B4XRect *)Rect1;

- (void)DrawTextRotated:(NSString *)Text :(float)x :(float)y :(B4XFont *)Font :(int)Color :(NSString *)Align :(float)Degrees;

- (void)DrawText:(NSString *)Text :(float)x :(float)y :(B4XFont *)Font :(int)Color :(NSString *)Align;

- (void)Release;

- (B4XRect *)MeasureText:(NSString *)Text :(B4XFont *)Font;
@end
//~shortname: B4XRect
//~objectwrapper: B4IRect*
@interface B4XRect : B4IObjectWrapper
@property (nonatomic) float Left;
@property (nonatomic) float Top;
@property (nonatomic) float Right;
@property (nonatomic) float Bottom;
@property (nonatomic)float Width;
@property (nonatomic)float Height;

//Returns the horizontal center.
@property (nonatomic, readonly) float CenterX;
//Returns the vertical center.
@property (nonatomic, readonly) float CenterY;
//Initializes a new Rect and sets its dimensions.
- (void)Initialize:(float)Left :(float)Top :(float)Right :(float)Bottom;

@end
//~Shortname:B4XPath
//~ObjectWrapper:UIBezierPath*
@interface B4XPath : B4IObjectWrapper

- (B4XPath *)Initialize:(float)x :(float)y;

- (B4XPath *)InitializeOval:(B4XRect *)Rect1;

- (B4XPath *)InitializeArc:(float)x :(float)y :(float)Radius :(float)StartingAngle :(float)SweepAngle;

- (B4XPath *)InitializeRoundedRect:(B4XRect *)Rect1 :(float)CornersRadius;

- (B4XPath *)LineTo:(float)x :(float)y;
@end